==> linux-nfs-0.4.19.tar.gz: linux-nfs-0.4.19/utils/mountd/auth.c <==
/*
 * utils/mountd/auth.c
 *
 * Authentication procedures for mountd.
 *
 * Copyright (C) 1995, 1996 Olaf Kirch <okir@monad.swb.de>
 */

#include <sys/stat.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "nfslib.h"
#include "exportfs.h"
#include "mountd.h"

static void	auth_fixpath(char *path);
static char	*export_file = NULL;

void
auth_init(char *exports)
{
	export_file = exports;
	auth_reload();
}

int
auth_reload()
{
	static time_t		last_modified = 0;
	struct stat		stb;

	if (stat(_PATH_XTAB, &stb) < 0)
		xlog(L_FATAL, "couldn't stat %s", _PATH_XTAB);
	if (stb.st_mtime == last_modified)
		return 0;
	last_modified = stb.st_atime;

	export_freeall();
	export_read(export_file);
	xtab_read();

	return 1;
}

nfs_export *
auth_authenticate(char *what, struct sockaddr_in *caller, char *path)
{
	struct in_addr		addr = caller->sin_addr;
	struct hostent		*hp;
	nfs_export		*exp;

	if (path[0] != '/') {
		xlog(L_WARNING, "bad path in %s request from %s: \"%s\"",
			what, inet_ntoa(addr), path);
		return NULL;
	}
	auth_fixpath(path);

	if (!(hp = gethostbyaddr((char *)&addr, sizeof(addr), AF_INET))) {
		xlog(L_WARNING, "%s request from unknown host %s",
				what, inet_ntoa(addr));
		return NULL;
	}
	if (!(exp = export_find(hp, path))) {
		xlog(L_WARNING, "refused %s request from %s: no export entry",
				what, hp->h_name);
		return NULL;
	}
	if (!exp->m_mayexport) {
		xlog(L_WARNING, "refused %s request from %s: not exported",
				what, hp->h_name);
		return NULL;
	}

	if (!(exp->m_export.e_flags & NFSEXP_INSECURE_PORT) &&
	    (ntohs(caller->sin_port) <  IPPORT_RESERVED/2 ||
	     ntohs(caller->sin_port) >= IPPORT_RESERVED)) {
		xlog(L_WARNING, "refused %s request from %s: illegal port %d",
				what, hp->h_name, ntohs(caller->sin_port));
		return NULL;
	}

	xlog(L_NOTICE, "authenticated %s request from %s",
				what, hp->h_name);
	return exp;
}

static void
auth_fixpath(char *path)
{
	char	*sp, *cp;

	for (sp = cp = path; *sp; sp++) {
		if (*sp != '/' || sp[1] != '/')
			*cp++ = *sp;
	}
	while (cp > path+1 && cp[-1] == '/')
		cp--;
	*cp = '\0';
}

==> linux-nfs-0.4.21.tar.gz: linux-nfs-0.4.21/utils/mountd/auth.c <==
/*
 * utils/mountd/auth.c
 *
 * Authentication procedures for mountd.
 *
 * Copyright (C) 1995, 1996 Olaf Kirch <okir@monad.swb.de>
 */

#include <sys/stat.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "nfslib.h"
#include "exportfs.h"
#include "mountd.h"

static void	auth_fixpath(char *path);
static char	*export_file = NULL;

void
auth_init(char *exports)
{
	export_file = exports;
	auth_reload();
}

int
auth_reload()
{
	static time_t		last_modified = 0;
	struct stat		stb;

	if (stat(_PATH_XTAB, &stb) < 0)
		xlog(L_FATAL, "couldn't stat %s", _PATH_XTAB);
	if (stb.st_mtime == last_modified)
		return 0;
	last_modified = stb.st_atime;

	export_freeall();
	// export_read(export_file);
	xtab_read();

	return 1;
}

nfs_export *
auth_authenticate(char *what, struct sockaddr_in *caller, char *path)
{
	struct in_addr		addr = caller->sin_addr;
	struct hostent		*hp;
	nfs_export		*exp;

	if (path[0] != '/') {
		xlog(L_WARNING, "bad path in %s request from %s: \"%s\"",
			what, inet_ntoa(addr), path);
		return NULL;
	}
	auth_fixpath(path);

	if (!(hp = gethostbyaddr((char *)&addr, sizeof(addr), AF_INET))) {
		xlog(L_WARNING, "%s request from unknown host %s",
				what, inet_ntoa(addr));
		return NULL;
	}
	if (!(exp = export_find(hp, path))) {
		xlog(L_WARNING, "refused %s request from %s: no export entry",
				what, hp->h_name);
		return NULL;
	}
	if (!exp->m_mayexport) {
		xlog(L_WARNING, "refused %s request from %s: not exported",
				what, hp->h_name);
		return NULL;
	}

	if (!(exp->m_export.e_flags & NFSEXP_INSECURE_PORT) &&
	    (ntohs(caller->sin_port) <  IPPORT_RESERVED/2 ||
	     ntohs(caller->sin_port) >= IPPORT_RESERVED)) {
		xlog(L_WARNING, "refused %s request from %s: illegal port %d",
				what, hp->h_name, ntohs(caller->sin_port));
		return NULL;
	}

	xlog(L_NOTICE, "authenticated %s request from %s",
				what, hp->h_name);
	return exp;
}

static void
auth_fixpath(char *path)
{
	char	*sp, *cp;

	for (sp = cp = path; *sp; sp++) {
		if (*sp != '/' || sp[1] != '/')
			*cp++ = *sp;
	}
	while (cp > path+1 && cp[-1] == '/')
		cp--;
	*cp = '\0';
}
